@@ -110,11 +110,19 @@ bool _AssociativeList::ParseStringRepresentation (_String& serialized_form, _For
                   key = *(_String*)key_value_pair(0);
                 }
                 
-                _Formula value      (*(_String*)key_value_pair(1),theP, doErrors?nil :fpc.errMsg());
-                HBLObjectRef   valueC  = compute_keys_values ? value.Compute() : new _MathObject;
-              
+                HBLObjectRef   valueC = nil;
+                if (compute_keys_values) {
+                    _Formula value      (*(_String*)key_value_pair(1),theP, doErrors?nil :fpc.errMsg());
+                    valueC  = value.Compute();
+                    if (valueC) {
+                        valueC->AddAReference();
+                    }
+                } else {
+                    valueC  =  new _MathObject;
+                }
+                
                 if (valueC) {
-                    MStore (key, valueC, compute_keys_values);
+                    MStore (key, valueC, false);
                 } else {
                     throw (((_String*)key_value_pair(1))->Enquote() & " could not be evaluated");
 
@@ -123,7 +131,7 @@ bool _AssociativeList::ParseStringRepresentation (_String& serialized_form, _For
                 throw (((_String*)splitKeys(k))->Enquote() & " does not appear to specify a valid key:value pair");
             }
         }
-    } catch (const _String err) {
+    } catch (const _String& err) {
         if (doErrors) {
             HandleApplicationError(err);
         }
@@ -156,12 +164,18 @@ void _AssociativeList::Duplicate (BaseRefConst br) {
 
 //_____________________________________________________________________________________________
 
-HBLObjectRef _AssociativeList::MCoord (HBLObjectRef p) {
+HBLObjectRef _AssociativeList::MCoord (HBLObjectRef p, HBLObjectRef cache) {
+    if (cache && cache->ObjectClass() == STRING) {
+        _StringBuffer * content = new _StringBuffer ((_String*)p->toStr());
+        //content->AddAReference();
+        ((_FString*)cache)->SetStringContent (content);
+        return cache;
+    }
     return new _FString ((_String*)p->toStr());
 }
 
 //_____________________________________________________________________________________________
-HBLObjectRef _AssociativeList::MAccess (HBLObjectRef p) {
+HBLObjectRef _AssociativeList::MAccess (HBLObjectRef p, HBLObjectRef cache) {
     long        f;
 
     if (p->ObjectClass() == STRING) {
@@ -172,7 +186,8 @@ HBLObjectRef _AssociativeList::MAccess (HBLObjectRef p) {
     }
     if (f>=0L) {
         HBLObjectRef res = (HBLObjectRef)avl.GetXtra (f);
-        res->AddAReference();
+        if (cache != res)
+            res->AddAReference();
         return res;
     } else {
         return kAssociativeListDefaultReturn;
@@ -199,12 +214,18 @@ bool _AssociativeList::Equal (HBLObjectRef p) {
 }
 
 //_____________________________________________________________________________________________
-HBLObjectRef _AssociativeList::Random (HBLObjectRef p) {
+HBLObjectRef _AssociativeList::Random (HBLObjectRef p, HBLObjectRef cache) {
     bool with_replacement = false;
     
     try {
         if (p->ObjectClass() == NUMBER) {
-            _AssociativeList * result = new _AssociativeList;
+            _AssociativeList * result;
+            if (cache && cache->ObjectClass() == ASSOCIATIVE_LIST) {
+                result = (_AssociativeList*)cache;
+                result->Clear();
+            } else {
+                result = new _AssociativeList;
+            }
             with_replacement = !CheckEqual(0.0, p->Compute()->Value());
             
             //unsigned long items = countitems();
@@ -234,14 +255,14 @@ HBLObjectRef _AssociativeList::Random (HBLObjectRef p) {
         } else {
             throw (_String ("Unsupported argument type"));
         }
-    } catch (const _String err) {
+    } catch (const _String& err) {
         HandleApplicationError (err);
     }
     return new _MathObject;
 }
 
 //_____________________________________________________________________________________________
-HBLObjectRef _AssociativeList::MIterator (HBLObjectRef p, HBLObjectRef p2) {
+HBLObjectRef _AssociativeList::MIterator (HBLObjectRef p, HBLObjectRef p2, HBLObjectRef cache) {
     
     const _String     kAVLIteratorOrder          = "INDEXORDER",
                       kAVLIteratorOrderValue     = "VALUEINDEXORDER";
@@ -279,6 +300,7 @@ HBLObjectRef _AssociativeList::MIterator (HBLObjectRef p, HBLObjectRef p2) {
                     actionFormula.GetList() < new _Operation()
                                             < new _Operation()
                                             < new _Operation(kEmptyString,-callback-1L);
+                    
 
                     if (filter >= 0L) {
                         testFormula.GetList() < new _Operation() < new _Operation(kEmptyString,-filter-1L);
@@ -299,6 +321,7 @@ HBLObjectRef _AssociativeList::MIterator (HBLObjectRef p, HBLObjectRef p2) {
                             }
                             actionFormula.GetIthTerm(0)->SetNumber(fKey);
                             actionFormula.GetIthTerm(1)->SetNumber((HBLObjectRef)filter_key_value.get_object());
+                            //StringToConsole((_String*)actionFormula.toStr(kFormulaStringConversionNormal)); NLToConsole();
                             actionFormula.Compute();
                             done ++;
                         }
@@ -325,7 +348,7 @@ HBLObjectRef _AssociativeList::MIterator (HBLObjectRef p, HBLObjectRef p2) {
                 if (index >= 0L) {
                   result = (*mode == kAVLIteratorOrder )? (new _FString(*(_String*)avl.Retrieve(index),false)): ((HBLObjectRef)avl.GetXtra (index)->makeDynamic());
                 } else {
-                  throw ("Index out of bounds in call to AVL iterator (by index)");
+                  throw _String("Index out of bounds in call to AVL iterator (by index)");
                 }
             }
           
@@ -333,12 +356,12 @@ HBLObjectRef _AssociativeList::MIterator (HBLObjectRef p, HBLObjectRef p2) {
               return result;
             }
         } else {
-            throw ("Both arguments must be Strings (or a String Literal and a number) in an iterator call for Associative Arrays");
+            throw _String("Both arguments must be Strings (or a String Literal and a number) in an iterator call for Associative Arrays");
         }
-    } catch (const _String err) {
+    } catch (const _String& err) {
         HandleApplicationError (err);
     }
-    return new _Constant (done);
+    return _returnConstantOrUseCache(done, cache);
 }
 
 //_____________________________________________________________________________________________
@@ -620,7 +643,7 @@ void        _AssociativeList::Merge (HBLObjectRef p) {
 }
 
   //_____________________________________________________________________________________________
-HBLObjectRef        _AssociativeList::ExtremeValue (bool do_mimimum) const {
+HBLObjectRef        _AssociativeList::ExtremeValue (bool do_mimimum, HBLObjectRef cache) const {
   _String const * best_key = nil;
   hyFloat best_value = do_mimimum ? INFINITY : -INFINITY;
   
@@ -655,7 +678,7 @@ HBLObjectRef        _AssociativeList::ExtremeValue (bool do_mimimum) const {
 }
 
 //_____________________________________________________________________________________________
-HBLObjectRef        _AssociativeList::Sum (void) {
+HBLObjectRef        _AssociativeList::Sum (HBLObjectRef cache) {
     hyFloat sum = 0.;
         
     for (AVLListXLIteratorKeyValue key_value : AVLListXLIterator (&avl)) {
@@ -668,31 +691,35 @@ HBLObjectRef        _AssociativeList::Sum (void) {
                 sum += ((_FString*)value)->get_str().to_float();
                 break;
             case MATRIX: {
-                _Constant * sumOfValue = (_Constant*) ((_Matrix*)value->Compute())->Sum();
+                _Constant * sumOfValue = (_Constant*) ((_Matrix*)value->Compute())->Sum(cache);
                 sum += sumOfValue->Value();
-                DeleteObject (sumOfValue);
+                if (sumOfValue != cache) {
+                    DeleteObject (sumOfValue);
+                }
                 break;
             }
             case ASSOCIATIVE_LIST: {
-                _Constant * sumOfValue = (_Constant*) ((_AssociativeList*)value->Compute())->Sum();
+                _Constant * sumOfValue = (_Constant*) ((_AssociativeList*)value->Compute())->Sum(cache);
                 sum += sumOfValue->Value();
-                DeleteObject (sumOfValue);
+                if (sumOfValue != cache) {
+                    DeleteObject (sumOfValue);
+                }
                 break;
             }
         }
     }
     
-    return new _Constant (sum);
+    return _returnConstantOrUseCache (sum, cache);
 }
 
 //__________________________________________________________________________________
 
 
-HBLObjectRef _AssociativeList::ExecuteSingleOp (long opCode, _List* arguments, _hyExecutionContext* context)  {
+HBLObjectRef _AssociativeList::ExecuteSingleOp (long opCode, _List* arguments, _hyExecutionContext* context, HBLObjectRef cache)  {
   
   switch (opCode) {
     case HY_OP_CODE_ABS:
-      return new _Constant (Length());
+     return _returnConstantOrUseCache(Length(), cache);
       
     case HY_OP_CODE_EVAL:
       return (HBLObjectRef) makeDynamic();
@@ -731,13 +758,13 @@ HBLObjectRef _AssociativeList::ExecuteSingleOp (long opCode, _List* arguments, _
     }
       
     case HY_OP_CODE_TYPE: // Type
-      return Type();
+      return Type(cache);
       
     case HY_OP_CODE_MAX: // Max
-      return ExtremeValue (false);
+      return ExtremeValue (false, cache);
       
     case HY_OP_CODE_MIN: // Max
-      return ExtremeValue (true);
+      return ExtremeValue (true, cache);
      
 
     
@@ -752,57 +779,57 @@ HBLObjectRef _AssociativeList::ExecuteSingleOp (long opCode, _List* arguments, _
         if (!MStore (new_key, arg0, true)) {
             DeleteObject (new_key);
         }
-        return new _Constant (avl.countitems());
+        return _returnConstantOrUseCache(avl.countitems(), cache);
       }
-      return Sum ();
+      return Sum (cache);
   }
   
 
   if (arg0) {
     switch (opCode) { // operations that require exactly one argument
        case HY_OP_CODE_EQ:
-        return new _Constant (Equal(arg0));
+        return _returnConstantOrUseCache (Equal(arg0), cache);
             
        case HY_OP_CODE_NEQ:
-        return new _Constant (!Equal(arg0));
+        return _returnConstantOrUseCache (!Equal(arg0), cache);
             
       case HY_OP_CODE_MCOORD: // MCoord
-        return MCoord (arg0);
+        return MCoord (arg0, cache);
         
       case HY_OP_CODE_MUL: // merge
         Merge (arg0);
-        return new _Constant (avl.countitems());
+        return _returnConstantOrUseCache (avl.countitems(), cache);
         
       case HY_OP_CODE_SUB:
         DeleteByKey (arg0);
-        return new _Constant (avl.countitems());
-        
+         return _returnConstantOrUseCache (avl.countitems(), cache);
+
         case HY_OP_CODE_RANDOM:
-            return Random (arg0);
+            return Random (arg0, cache);
 
         case HY_OP_CODE_DIV:
         
         if (arg0->ObjectClass () == STRING) {
           if (avl.Find (&((_FString*)arg0)->get_str()) >= 0L) {
-            return new _Constant (1.0);
+            return _returnConstantOrUseCache (1., cache);
           }
         } else {
           _String serialized ((_String*)arg0->toStr());
           if (avl.Find (&serialized) >= 0L) {
-            return new _Constant (1.0);
+            return _returnConstantOrUseCache (1., cache);
           }
         }
-        return new _Constant (0.0);
-        
+        return _returnConstantOrUseCache (0., cache);
+
     }
     _MathObject * arg1 = _extract_argument (arguments, 1UL, false);
     
     switch (opCode) { //  check operations with 1 or 2 arguments
       case HY_OP_CODE_MACCESS: // MAccess
         if (arg1) {
-          return MIterator (arg0,arg1);
+          return MIterator (arg0,arg1, cache);
         } else {
-          return MAccess   (arg0);
+          return MAccess   (arg0, cache);
         }
     }
   }
