@@ -261,7 +261,7 @@ void _CategoryVariable::Construct (_List& parameters, _VariableContainer *theP)
                     if(scannedVarsList.lLength==1) {
                         if (scannedVarsList[0]==hy_n_variable->get_index()) {
                               for (unsigned long i=0UL; i<intervals; i++) {
-                                hy_n_variable->SetValue(new _Constant ((hyFloat)i), false);
+                                hy_n_variable->SetValue(new _Constant ((hyFloat)i), false, true, NULL);
                                 (*weights)[i]= probabilities.Compute()->Value();
                             }
                             check = checkWeightMatrix (*weights);
@@ -499,54 +499,56 @@ void _CategoryVariable::Construct (_List& parameters, _VariableContainer *theP)
         Parse    (&meanC,*param,fpc, nil);
 
         if (parameters.lLength>8) {
-            _String hmmModelName = AppendContainerName(*(_String*)parameters(8),theP);
-            f = FindModelName(hmmModelName);
-            if (f==-1) {
-                if (constantOnPartition == *(_String*)parameters (8)) {
-                    flags = CONSTANT_ON_PARTITION;
-                } else {
-                    HandleApplicationError (errorMsg & (*(_String*)parameters(8))& " is not an existing model identifier in call to 'category'");
-                    return;
-                }
-            } else {
-                if (covariantVar) {
-                    HandleApplicationError (errorMsg & "Non-independent random variables can't also be hidden Markov.");
-                    return;
-                }
-                long mindex = f;
-                _Matrix * hmm,
-                        * freq;
-
-                bool    mbf;
-
-                RetrieveModelComponents (mindex, hmm, freq, mbf);
-                mbf = false;
-
-                if (hmm) {
-
-                    f = weights->GetHDim()*weights->GetVDim();
-
-                    if (hmm->GetHDim()==f && hmm->GetVDim()==f) {
-                        _SimpleList   hmmVars,
-                                      existingVars (parameterList);
-
-                        _AVLList      sv (&hmmVars);
-
-                        hmm->ScanForVariables (sv,true);
-                        freq->ScanForVariables (sv,true);
-
-
-                        sv.ReorderList();
-                        parameterList.Union (hmmVars,existingVars);
-                        exclude.Subtract (hmmVars,existingVars);
-
-                        hiddenMarkovModel = mindex;
-                        mbf = true;
+            if (((_String*)parameters(8))->nonempty()) {
+                _String hmmModelName = AppendContainerName(*(_String*)parameters(8),theP);
+                f = FindModelName(hmmModelName);
+                if (f==-1) {
+                    if (constantOnPartition == *(_String*)parameters (8)) {
+                        flags = CONSTANT_ON_PARTITION;
+                    } else {
+                        HandleApplicationError (errorMsg & (*(_String*)parameters(8))& " is not an existing model identifier in call to 'category'");
+                        return;
                     }
-                }
+                } else {
+                    if (covariantVar) {
+                        HandleApplicationError (errorMsg & "Non-independent random variables can't also be hidden Markov.");
+                        return;
+                    }
+                    long mindex = f;
+                    _Matrix * hmm,
+                            * freq;
 
-                if (!mbf) {
-                    HandleApplicationError (errorMsg & (*(_String*)parameters(8))& " is not a valid HMM-component model (square matrix of dimension "&_String (f) & ") identifier in call to 'category'");
+                    bool    mbf;
+
+                    RetrieveModelComponents (mindex, hmm, freq, mbf);
+                    mbf = false;
+
+                    if (hmm) {
+
+                        f = weights->GetHDim()*weights->GetVDim();
+
+                        if (hmm->GetHDim()==f && hmm->GetVDim()==f) {
+                            _SimpleList   hmmVars,
+                                          existingVars (parameterList);
+
+                            _AVLList      sv (&hmmVars);
+
+                            hmm->ScanForVariables (sv,true);
+                            freq->ScanForVariables (sv,true);
+
+
+                            sv.ReorderList();
+                            parameterList.Union (hmmVars,existingVars);
+                            exclude.Subtract (hmmVars,existingVars);
+
+                            hiddenMarkovModel = mindex;
+                            mbf = true;
+                        }
+                    }
+
+                    if (!mbf) {
+                        HandleApplicationError (errorMsg & (*(_String*)parameters(8))& " is not a valid HMM-component model (square matrix of dimension "&_String (f) & ") identifier in call to 'category'");
+                    }
                 }
             }
         }
@@ -741,7 +743,7 @@ hyFloat  _CategoryVariable::SetIntervalValue (long ival, bool recalc)
     } else {
         newIntervalValue = ((_Matrix*)values->RetrieveNumeric())->theData[ival];
     }
-    SetValue (new _Constant(newIntervalValue),false);
+    SetValue (new _Constant(newIntervalValue),false,true,NULL);
     /*if (ival == 0) {
         printf ("\n\n");
     }
@@ -1053,10 +1055,10 @@ bool        _CategoryVariable::UpdateIntervalsAndValues (bool force) {
                         values->theData[i] = density.MeanIntegral (hy_x_variable,currentLeft,(*intervalEnds)[i])/(*ew)[i];
                     } else {
                         _Constant    currentRight ((*intervalEnds)[i]);
-                        hy_x_variable->SetValue(&currentRight);
+                        hy_x_variable->SetValue(&currentRight, true, true, NULL);
                         values->theData[i] = meanC.Compute()->Value();
                         currentRight.SetValue(currentLeft);
-                        hy_x_variable->SetValue(&currentRight);
+                        hy_x_variable->SetValue(&currentRight, true, true, NULL);
                         values->theData[i] = x_min+((*values)[i]- meanC.Compute()->Value())/(*ew)[i];
                         if (values->theData[i]>x_max) {
                             values->theData[i] = x_max;
@@ -1093,10 +1095,10 @@ bool        _CategoryVariable::UpdateIntervalsAndValues (bool force) {
                     (*values)[i] = density.MeanIntegral (hy_x_variable,currentLeft,(*intervalEnds)[i],true)/(*ew)[i];
                 } else {
                     _Constant    currentRight (x_max);
-                    hy_x_variable->SetValue(&currentRight);
+                    hy_x_variable->SetValue(&currentRight, true, false, NULL);
                     values->theData[i] = meanC.Compute()->Value();
                     currentRight.SetValue(currentLeft);
-                    hy_x_variable->SetValue(&currentRight);
+                    hy_x_variable->SetValue(&currentRight, true, false, NULL);
                     values->theData[i] = x_min+((*values)[i]- meanC.Compute()->Value())/(*ew)[i];
                     if (values->theData[i]>x_max) {
                         values->theData[i] = x_max;
@@ -1129,8 +1131,7 @@ bool        _CategoryVariable::UpdateIntervalsAndValues (bool force) {
             if (meanC.IsEmpty()) {
                 distMean = density.MeanIntegral (hy_x_variable,x_min,x_max, true);
             } else {
-                _Constant    currentRight (x_max);
-                hy_x_variable->SetValue(&currentRight);
+                hy_x_variable->SetValue(new _Constant (x_max), false, false, NULL);
                 distMean = meanC.Compute()->Value();
             }
 
