@@ -272,7 +272,8 @@ namespace hy_global {
   terminate_execution,
   has_terminal_stdout,
   has_terminal_stderr,
-  ignore_kw_defaults;
+  ignore_kw_defaults,
+  force_verbosity_from_cli;
   
   extern  int      hy_mpi_node_rank,
   hy_mpi_node_count;
@@ -307,8 +308,8 @@ namespace hy_global {
   kErrorStringUnterminatedMatrix,
   kNoneToken,
   kNullToken,
-  kNoKWMatch
-  ;
+  kNoKWMatch,
+  kEndIteration;
   
   extern  _String  hy_base_directory,
   hy_error_log_name,
@@ -320,7 +321,7 @@ namespace hy_global {
   
   extern _Variable * hy_x_variable, * hy_n_variable;
     
-  extern hyFloat     kMachineEpsilon;
+  extern const hyFloat     kMachineEpsilon;
   
   extern regex_t   * hy_float_regex,
                    * hy_replicate_constraint_regexp;
