@@ -40,7 +40,6 @@
 #include <math.h>
 #include <float.h>
 #include <limits.h>
-#include "stdlib.h"
 #include "string.h"
 #include <stdio.h>
 #include <time.h>
@@ -114,8 +113,6 @@ hyFloat  tolerance  = DBL_EPSILON;
 //__________________________________________________________________________________
 //SW: Helper functions
 
-//__________________________________________________________________________________
-void            DeleteTreeVariable      (long, _SimpleList &, bool);
 
 
 
@@ -351,9 +348,7 @@ void       UpdateChangingFlas (long vN) {
 }
 
 //__________________________________________________________________________________
-void       UpdateChangingFlas (_SimpleList & involvedVariables)
-{
-
+void       UpdateChangingFlas (_SimpleList const & involvedVariables) {
     long          topLimit         = compiledFormulaeParameters.lLength;
     _SimpleList * toDelete         = nil;
 
@@ -407,7 +402,7 @@ void DeleteVariable (long dv, bool deleteself, bool do_checks) {
                     if (self_variable != check_variable && check_variable->CheckFForDependence (vidx,false)) {
                         HBLObjectRef current_variable_value = check_variable->Compute();
                         current_variable_value->AddAReference(); // if this isn't done; the object will be deleted when the formula is cleared in SetValue
-                        check_variable->SetValue (current_variable_value);
+                        check_variable->SetValue (current_variable_value,true,true,NULL);
                         DeleteObject (current_variable_value);
                     }
                 }
@@ -447,17 +442,131 @@ void DeleteVariable (long dv, bool deleteself, bool do_checks) {
 }
 
 //__________________________________________________________________________________
-void DeleteTreeVariable (long dv, _SimpleList & parms, bool doDeps)
-{
-    if (dv>=0) {
-        _String *name  = (_String*)variableNames.Retrieve (dv);
-        _String myName = *name&".";
-        long    vidx   = variableNames.GetXtra (dv);
+void DeleteTreeVariable (long tree_variable_index, _SimpleList & parms, _String * prefix, bool leftovers) {
+    if (tree_variable_index>=0) {
+        
+        /*
+            1. Indentify all the variables that are in the tree.* and tree.*.* domain
+            2. Scan all the variables that are dependend on the list here and "batch" unconstrain them
+            3. Delete the tree and tree nodes; store the remainign variables in parms
+        */
+        
+        
+        _String const *tree_name  = prefix ? prefix : (_String const*)variableNames.Retrieve (tree_variable_index),
+                       kDot = ".",
+                       tree_prefix = *tree_name&kDot;
+        
+        _SimpleList traversal_stack,
+                    indices_to_delete,
+                    all_subdomain_indices;
+        
+        _AVLList    indices_to_delete_avl (&indices_to_delete),
+                    all_subdomain_indices_avl (&all_subdomain_indices),
+                    will_keep_these (&parms);
+                    
+        
+        long tree_array_index = tree_variable_index;
+        variableNames.Find (tree_name,traversal_stack);
+        
+        while (long next_var = variableNames.Next (tree_array_index,traversal_stack)) {
+            if (next_var < 0) {
+                break;
+            }
+            _String const * next_var_name = (_String const *)variableNames.Retrieve(next_var);
+            if (next_var_name->BeginsWith(tree_prefix)) {
+                all_subdomain_indices_avl.InsertNumber(next_var);
+                
+                _Variable * secondary_variable = FetchVar (next_var);
+                                
+                if (next_var_name->FindBackwards(kDot, tree_prefix.length() + 1, kStringEnd) != kNotFound && !secondary_variable->IsContainer()) {
+                    // 20200220: deleting all containers here; only "plain" variables will be kept
+                    // subnode level
+                    will_keep_these.InsertNumber(next_var);
+                 } else {
+                    //printf ("Deleting variable %s\n" , next_var_name->get_str());
+                    indices_to_delete_avl.InsertNumber (next_var);
+                }
+            } else {
+                break;
+            }
+            tree_array_index = next_var;
+        }
+        
+        //if ((nvid = variableNames.Next (dv,recCache))>=0)
+        
+        all_subdomain_indices_avl.ReorderList();
+        indices_to_delete_avl.ReorderList();
+        will_keep_these.ReorderList();
+        
+        _SimpleList _touched_dependent_variables,
+                    touched_containers;
+        
+        _AVLList    touched_dependent_variables (&_touched_dependent_variables);
+        
+        DoForEachVariable([&touched_dependent_variables, &indices_to_delete_avl, &all_subdomain_indices_avl] (_Variable* v, long idx) -> void {
+            // only care about variables that are not going in the subdomain
+            if (all_subdomain_indices_avl.FindLong(idx) < 0) {
+                if (v->IsContainer()) {
+                    //_VariableContainer * vc = (_VariableContainer*)vc;
+                    /* TODO, it is theoretically possible that a node / tree variable is a template variable
+                       in another container, but there are no use cases of this;
+                       need to check, but later
+                     */
+                } else {
+                    if (v->CheckFForDependence(indices_to_delete_avl)) {
+                        //printf ("Touched dependent variable %s\n" , v->GetName()->get_str());
+                        touched_dependent_variables.InsertNumber(v->get_index());
+                    }
+                }
+            }
+        });
+        
 
-        UpdateChangingFlas (vidx);
+        
+        //SetVariableToOwn
+        
+        UpdateChangingFlas(all_subdomain_indices);
+        // reset compiled formulas that may contain variable indices being deleted
+        
+        if (leftovers) {
+            SetVariablesToOwnValues (will_keep_these);
+        } else {
+            indices_to_delete << parms;
+        }
+        
+        if (touched_dependent_variables.countitems()) {
+            SetVariablesToOwnValues (touched_dependent_variables);
+        }
+        
+        /*if (prefix) {
+            printf ("CLEARING %d variables\n", indices_to_delete.countitems() );
+        }*/
+        
+        indices_to_delete.Each ([] (long var_idx, unsigned long) -> void {
+            _Variable * delvar = LocateVar (var_idx);
+            //printf ("Deleting variable %s\n" , delvar->GetName()->get_str());
+            if (delvar->ObjectClass() != TREE) {
+                variableNames.Delete (delvar->GetName(),true);
+                (*((_SimpleList*)&variablePtrs))[delvar->get_index()]=0;
+                freeSlots<<delvar->get_index();
+                DeleteObject (delvar);
+            } else {
+                ((_VariableContainer*)delvar)->Clear();
+            }
+        });
+        
+
+        return;
+         
+        /*_String const *tree_name  = (_String const*)variableNames.Retrieve (dv),
+                       tree_prefix = *tree_name&".";
+        
+        long    variable_index   = variableNames.GetXtra (dv);
+
+        UpdateChangingFlas (variable_index);
 
         _SimpleList recCache;
-        variableNames.Find (name,recCache);
+        variableNames.Find (tree_name,recCache);
         _String     nextVarID;
         long        nvid;
         if ((nvid = variableNames.Next (dv,recCache))>=0) {
@@ -465,6 +574,7 @@ void DeleteTreeVariable (long dv, _SimpleList & parms, bool doDeps)
         }
 
 
+        
         {
             _SimpleList tcache;
             long        iv,
@@ -473,11 +583,11 @@ void DeleteTreeVariable (long dv, _SimpleList & parms, bool doDeps)
             for (; k>=0; k = variableNames.Traverser (tcache, iv)) {
                 _Variable * thisVar = FetchVar(k);
 
-                if (thisVar->CheckFForDependence (vidx,false)) {
+                if (thisVar->CheckFForDependence (variable_index,false)) {
                     HBLObjectRef curValue = thisVar->Compute();
                     curValue->AddAReference();
-                    thisVar->SetValue (curValue);
-                    DeleteObject (curValue);
+                    thisVar->SetValue (curValue, false);
+                    //DeleteObject (curValue);
                 }
             }
         }
@@ -485,8 +595,8 @@ void DeleteTreeVariable (long dv, _SimpleList & parms, bool doDeps)
         _Variable* delvar = (FetchVar(dv));
         if (delvar->ObjectClass() != TREE) {
             variableNames.Delete (variableNames.Retrieve(dv),true);
-            (*((_SimpleList*)&variablePtrs))[vidx]=0;
-            freeSlots<<vidx;
+            (*((_SimpleList*)&variablePtrs))[variable_index]=0;
+            freeSlots<<variable_index;
             DeleteObject (delvar);
         } else {
             ((_VariableContainer*)delvar)->Clear();
@@ -497,8 +607,8 @@ void DeleteTreeVariable (long dv, _SimpleList & parms, bool doDeps)
             long nextVar = variableNames.Find (&nextVarID,recCache);
             for (; nextVar>=0; nextVar = variableNames.Next (nextVar, recCache)) {
                 _String dependent = *(_String*)variableNames.Retrieve (nextVar);
-                if (dependent.BeginsWith(myName)) {
-                    if (dependent.Find ('.', myName.length()+1, -1)>=0) {
+                if (dependent.BeginsWith(tree_prefix)) {
+                    if (dependent.Find ('.', tree_prefix.length()+1, -1)>=0) {
                         _Variable * checkDep = FetchVar (nextVar);
                         if (!checkDep->IsIndependent()) {
                             HBLObjectRef curValue = checkDep->Compute();
@@ -518,9 +628,9 @@ void DeleteTreeVariable (long dv, _SimpleList & parms, bool doDeps)
             for (unsigned long k=0; k<toDelete.lLength; k++) {
                 //StringToConsole (*(_String*)toDelete(k));
                 //BufferToConsole ("\n");
-                DeleteTreeVariable (*(_String*)toDelete(k),parms,false);
+                DeleteTreeVariable (LocateVarByName(*(_String*)toDelete(k)),parms,false);
             }
-        }
+        }*/
     }
 }
 //__________________________________________________________________________________
@@ -528,11 +638,6 @@ void DeleteVariable (_String const &name, bool deleteself) {
     DeleteVariable(LocateVarByName (name), deleteself);
 }
 
-//__________________________________________________________________________________
-void DeleteTreeVariable (_String&name, _SimpleList& parms, bool doDeps)
-{
-    DeleteTreeVariable(LocateVarByName (name), parms,doDeps);
-}
 
 //__________________________________________________________________________________
 _Variable* CheckReceptacle (_String const * name, _String const & fID, bool checkValid, bool isGlobal, bool clear_trees) {
@@ -555,7 +660,7 @@ _Variable* CheckReceptacle (_String const * name, _String const & fID, bool chec
       }
     }
 
-    if ( f<0L ) {
+    {
         _Variable dummy (*name, isGlobal);
         f = LocateVarByName (*name);
     }
@@ -583,7 +688,7 @@ _Variable* CheckReceptacleCommandIDException (_String const* name, const long id
     }
   }
   
-  if (f<0L) {
+  {
     _Variable (*name, isGlobal);
     f = LocateVarByName (*name);
   }
@@ -611,7 +716,7 @@ bool CheckReceptacleCommandIDAndStore (_String const* name, const long id, bool
 {
     _Variable *theV = CheckReceptacleCommandID (name, id, checkValid, isGlobal);
     if (theV) {
-        theV->SetValue (v, dup);
+        theV->SetValue (v, dup, true, NULL);
         return true;
     }
     if (!dup) {
@@ -625,7 +730,7 @@ bool CheckReceptacleCommandIDAndStore (_String const* name, const long id, bool
 bool CheckReceptacleAndStore (_String const* name, _String fID, bool checkValid, HBLObjectRef v, bool dup) {
     _Variable * theV = CheckReceptacle(name, fID, checkValid);
     if (theV) {
-        theV->SetValue (v, dup);
+        theV->SetValue (v, dup, true, NULL);
         return true;
     }
     if (!dup) {
